package org.xbib.jdbc.pool;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collection;
import java.util.List;
import org.xbib.jdbc.pool.api.XbibDataSource.FlushMode;
import org.xbib.jdbc.pool.api.XbibDataSourceListener;
import org.xbib.jdbc.pool.api.XbibDataSourceMetrics;
import org.xbib.jdbc.pool.api.XbibPoolInterceptor;
import org.xbib.jdbc.pool.api.configuration.XbibConnectionPoolConfiguration;
import org.xbib.jdbc.pool.api.configuration.XbibDataSourceConfiguration.MetricsEnabledListener;
import org.xbib.jdbc.pool.api.transaction.TransactionIntegration.ResourceRecoveryFactory;

public interface Pool extends MetricsEnabledListener, AutoCloseable, ResourceRecoveryFactory {

    void init();

    Connection getConnection() throws SQLException;

    XbibConnectionPoolConfiguration getConfiguration();

    XbibDataSourceMetrics getMetrics();

    XbibDataSourceListener[] getListeners();

    List<XbibPoolInterceptor> getPoolInterceptors();

    void setPoolInterceptors(Collection<? extends XbibPoolInterceptor> list);

    void returnConnectionHandler(ConnectionHandler handler) throws SQLException;

    void flushPool(FlushMode mode);

    boolean isHealthy(boolean newConnection) throws SQLException;

    int defaultJdbcIsolationLevel();

    @Override
    void close();

    long activeCount();

    long maxUsedCount();

    long availableCount();

    long awaitingCount();

    void resetMaxUsedCount();
}
