package org.xbib.jdbc.pool.api.configuration;

/**
 * Configuration of an XbibDataSource.
 */
public interface XbibDataSourceConfiguration {

    /**
     * Configuration of the pool of this DataSource
     */
    XbibConnectionPoolConfiguration connectionPoolConfiguration();

    /**
     * Actual XbibDataSource implementation. This allows flexibility to have different implementations of the API.
     */
    DataSourceImplementation dataSourceImplementation();

    /**
     * If this XbibDataSource collects metrics.
     */
    boolean metricsEnabled();

    /**
     * Enables or disables the collection of metrics.
     */
    void setMetricsEnabled(boolean metricsEnabled);

    /**
     * Allows registration of a callback to be invoked each time there is a change on the status of metrics collection.
     */
    void registerMetricsEnabledListener(MetricsEnabledListener metricsEnabledListener);

    /**
     * Available implementations of XbibDataSource.
     */
    enum DataSourceImplementation {

        /**
         * Xbib - the database connection pool
         */
        XBIB("org.xbib.jdbc.pool.DataSource"),

        /**
         * Specialization of the Xbib pool for the flush-on-close use case.
         */
        XBIB_POOLLESS("org.xbib.jdbc.pool.DataSource");

        private final String className;

        DataSourceImplementation(String className) {
            this.className = className;
        }

        public String className() {
            return className;
        }
    }

    interface MetricsEnabledListener {
        void onMetricsEnabled(boolean metricsEnabled);
    }
}
