package org.xbib.jdbc.pool.api.security;

import java.util.Properties;
import javax.security.auth.kerberos.KerberosTicket;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.ietf.jgss.Oid;

/**
 * Handle objects of type {@link GSSCredential} and {@link KerberosTicket}
 */
public class XbibKerberosSecurityProvider implements XbibSecurityProvider {

    private static final Properties EMPTY_PROPERTIES = new Properties();

    private static final String KERBEROS_v5 = "1.2.840.113554.1.2.2";

    public XbibKerberosSecurityProvider() {
    }

    @Override
    public Properties getSecurityProperties(Object securityObject) {
        if (securityObject instanceof GSSCredential) {
            try {
                Properties properties = new Properties();
                properties.setProperty("user", ((GSSCredential) securityObject).getName(new Oid(KERBEROS_v5)).toString());
                return properties;
            } catch (GSSException e) {
                // nothing we can do
                return EMPTY_PROPERTIES;
            }
        }
        if (securityObject instanceof KerberosTicket) {
            return EMPTY_PROPERTIES;
        }
        return null;
    }
}
