package org.xbib.jdbc.pool.util;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;
import org.xbib.jdbc.pool.ConnectionHandler;
import org.xbib.jdbc.pool.api.XbibPoolInterceptor;

@SuppressWarnings({"UtilityClass", "ObjectAllocationInLoop"})
public final class InterceptorHelper {

    private InterceptorHelper() {
    }

    public static void fireOnConnectionCreateInterceptor(List<? extends XbibPoolInterceptor> interceptors, ConnectionHandler handler) throws SQLException {
        if (interceptors != null && !interceptors.isEmpty()) {
            for (XbibPoolInterceptor interceptor : interceptors) {
                try (Connection connection = handler.detachedWrapper()) {
                    interceptor.onConnectionCreate(connection);
                }
            }
        }
    }

    public static void fireOnConnectionAcquiredInterceptor(List<? extends XbibPoolInterceptor> interceptors, ConnectionHandler handler) throws SQLException {
        if (interceptors != null && !interceptors.isEmpty()) {
            for (XbibPoolInterceptor interceptor : interceptors) {
                try (Connection connection = handler.detachedWrapper()) {
                    interceptor.onConnectionAcquire(connection);
                }
            }
        }
    }

    public static void fireOnConnectionReturnInterceptor(List<? extends XbibPoolInterceptor> interceptors, ConnectionHandler handler) throws SQLException {
        if (interceptors != null && !interceptors.isEmpty()) {
            for (int i = interceptors.size(); i > 0; ) {
                try (Connection connection = handler.detachedWrapper()) {
                    interceptors.get(--i).onConnectionReturn(connection);
                }
            }
        }
    }

    public static void fireOnConnectionDestroyInterceptor(List<? extends XbibPoolInterceptor> interceptors, ConnectionHandler handler) throws SQLException {
        if (interceptors != null && !interceptors.isEmpty()) {
            for (int i = interceptors.size(); i > 0; ) {
                try (Connection connection = handler.detachedWrapper()) {
                    interceptors.get(--i).onConnectionDestroy(connection);
                }
            }
        }
    }
}
