package org.xbib.jdbc.pool.util;

import java.sql.Connection;
import java.util.Arrays;
import org.xbib.jdbc.pool.ConnectionHandler;
import org.xbib.jdbc.pool.api.XbibDataSourceListener;

@SuppressWarnings("UtilityClass")
public final class ListenerHelper {

    private ListenerHelper() {
    }

    public static void fireBeforeConnectionCreation(XbibDataSourceListener[] listeners) {
        for (XbibDataSourceListener listener : listeners) {
            listener.beforeConnectionCreation();
        }
    }

    public static void fireOnConnectionCreation(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onConnectionCreation(handler.rawConnection());
        }
    }

    public static void fireOnConnectionPooled(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onConnectionPooled(handler.rawConnection());
        }
    }

    public static void fireBeforeConnectionAcquire(XbibDataSourceListener[] listeners) {
        for (XbibDataSourceListener listener : listeners) {
            listener.beforeConnectionAcquire();
        }
    }

    public static void fireOnConnectionAcquired(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onConnectionAcquire(handler.rawConnection());
        }
    }

    public static void fireBeforeConnectionReturn(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.beforeConnectionReturn(handler.rawConnection());
        }
    }

    public static void fireOnConnectionReturn(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onConnectionReturn(handler.rawConnection());
        }
    }

    public static void fireBeforeConnectionLeak(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.beforeConnectionLeak(handler.rawConnection());
        }
    }

    @SuppressWarnings({"StringConcatenation", "ObjectAllocationInLoop"})
    public static void fireOnConnectionLeak(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            Connection connection = handler.rawConnection();
            listener.onConnectionLeak(connection, handler.getHoldingThread());
            if (handler.getAcquisitionStackTrace() != null) {
                if (handler.isEnlisted()) {
                    listener.onInfo("Leaked connection " + connection + " is enlisted. Please make sure the associated transaction completes.");
                } else {
                    listener.onInfo("Leaked connection " + connection + " is not enlisted. To return it to the pool use the flush(LEAK) operation.");
                }
                listener.onInfo("Leaked connection " + connection + " acquired at: " + Arrays.toString(handler.getAcquisitionStackTrace()));
            }
            if (handler.getConnectionOperations() != null) {
                listener.onInfo("Operations executed on leaked connection " + connection + ": " + String.join(", ", handler.getConnectionOperations()));
            }
            if (handler.getLastOperationStackTrace() != null) {
                listener.onInfo("Stack trace of last executed operation on " + connection + ": " + Arrays.toString(handler.getLastOperationStackTrace()));
            }
            if (handler.getConnectionOperations() != null && handler.getConnectionOperations().contains("unwrap(Class<T>)")) {
                listener.onWarning("A possible cause for the leak of connection " + connection + " is a call to the unwrap() method. close() needs to be called on the connection object provided by the pool.");
            }
        }
    }

    public static void fireBeforeConnectionValidation(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.beforeConnectionValidation(handler.rawConnection());
        }
    }

    public static void fireOnConnectionValid(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onConnectionValid(handler.rawConnection());
        }
    }

    public static void fireOnConnectionInvalid(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onConnectionInvalid(handler.rawConnection());
        }
    }

    public static void fireBeforeConnectionFlush(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.beforeConnectionFlush(handler.rawConnection());
        }
    }

    public static void fireOnConnectionFlush(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onConnectionFlush(handler.rawConnection());
        }
    }

    public static void fireBeforeConnectionReap(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.beforeConnectionReap(handler.rawConnection());
        }
    }

    public static void fireOnConnectionReap(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onConnectionReap(handler.rawConnection());
        }
    }

    public static void fireBeforeConnectionDestroy(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.beforeConnectionDestroy(handler.rawConnection());
        }
    }

    public static void fireOnConnectionDestroy(XbibDataSourceListener[] listeners, ConnectionHandler handler) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onConnectionDestroy(handler.rawConnection());
        }
    }

    public static void fireOnWarning(XbibDataSourceListener[] listeners, String message) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onWarning(message);
        }
    }

    public static void fireOnWarning(XbibDataSourceListener[] listeners, Throwable throwable) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onWarning(throwable);
        }
    }

    public static void fireOnInfo(XbibDataSourceListener[] listeners, String message) {
        for (XbibDataSourceListener listener : listeners) {
            listener.onInfo(message);
        }
    }
}
